"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ethValidateSig = exports.ethSignWithPrivate = exports.ethPrivateToPublic = exports.MockEmitter = exports.MockGuardians = void 0;
const index_js_1 = require("../../index.js");
const utils_js_1 = require("../../utils.js");
class MockGuardians {
    setIndex;
    signers;
    constructor(setIndex, keys) {
        this.setIndex = setIndex;
        this.signers = keys.map((key, index) => {
            return { index, key };
        });
    }
    getPublicKeys() {
        return this.signers.map((guardian) => ethPrivateToPublic(guardian.key));
    }
    addSignatures(message, guardianIndices) {
        if (guardianIndices.length == 0)
            throw Error("guardianIndices.length == 0");
        const signers = this.signers.filter((signer) => guardianIndices.includes(signer.index));
        const vaa = (0, index_js_1.deserialize)("Uint8Array", message);
        for (let i = 0; i < signers.length; ++i) {
            const signer = signers.at(i);
            if (!signer)
                throw Error("No signer with index: " + i);
            const signature = ethSignWithPrivate(signer.key, (0, utils_js_1.keccak256)(vaa.hash));
            const s = new index_js_1.Signature(signature.r, signature.s, signature.recovery);
            // @ts-ignore -- wants it to be immutable
            vaa.signatures.push({ guardianIndex: i, signature: s });
        }
        return vaa;
    }
}
exports.MockGuardians = MockGuardians;
class MockEmitter {
    chain;
    address;
    sequence;
    constructor(emitterAddress, chain, startSequence) {
        this.chain = chain;
        this.address = emitterAddress;
        this.sequence = startSequence == undefined ? 0n : startSequence;
    }
    publishMessage(nonce, payload, consistencyLevel, timestamp, uptickSequence = true) {
        if (uptickSequence) {
            ++this.sequence;
        }
        return (0, index_js_1.serialize)((0, index_js_1.createVAA)("Uint8Array", {
            guardianSet: 0,
            signatures: [],
            nonce: nonce,
            timestamp: timestamp ?? 0,
            sequence: this.sequence,
            emitterChain: this.chain,
            emitterAddress: this.address,
            consistencyLevel: consistencyLevel,
            payload: payload,
        }));
    }
}
exports.MockEmitter = MockEmitter;
function ethPrivateToPublic(privateKey) {
    return utils_js_1.secp256k1.getPublicKey(privateKey);
}
exports.ethPrivateToPublic = ethPrivateToPublic;
function ethSignWithPrivate(privateKey, hash) {
    if (hash.length != 32)
        throw new Error("hash.length != 32");
    return utils_js_1.secp256k1.sign(hash, privateKey);
}
exports.ethSignWithPrivate = ethSignWithPrivate;
function ethValidateSig(signature, publicKey, hash) {
    const { r, s } = signature;
    return utils_js_1.secp256k1.verify({ r, s }, hash, publicKey);
}
exports.ethValidateSig = ethValidateSig;
//# sourceMappingURL=guardian.js.map